_G["addonName"] = "AISY"
_G["addonDesc"] = "Am I Saved Yet?"
_G["defaultDungeons"] = _G["defaultDungeons"] or {
    ["Eye of Azshara"] = true,
    ["Vault of the Wardens"] = true,
    ["Assault on Violet Hold"] = true,
    ["Court of Stars"] = true,
    ["Darkheart Thicket"] = true,
    ["Halls of Valor"] = true,
    ["The Arcway"] = true,
    ["Maw of Souls"] = true,
    ["Black Rook Hold"] = true,
    ["Neltharion's Lair"] = true
} -- TODO: External config? localized

local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)
local addon = LibStub("AceAddon-3.0"):NewAddon(_G["addonName"], "AceConsole-3.0", "AceEvent-3.0")

if LDB then
    AISYLauncher = LDB:NewDataObject(_G["addonName"], {
        type = "launcher",
        icon = "Interface\\AddOns\\right.tga",
        OnClick = function(clickedframe, button)
            if button == "RightButton" then
                _G.InterfaceOptionsFrame_OpenToCategory(addon.optionsFrame)
            elseif button == "LeftButton" then
                -- TODO: keep window open and allow instances to be selected / show loot? / show upgrades?
            end
        end,
        OnTooltipShow = function(tt)
            tt:AddLine("Monitored Dungeons") -- TODO: Localized 
            for key,value in pairs(addon:GetMonitoredDungeons()) do
                tt:AddLine((value and "|cff00FF00" or "|cffFF0000")..key.."|r") -- TODO: Localized / colors? / yellow perhaps for raids?
            end
        end
    })
end

function addon:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("aisyDB", {
        profile = {
            minimap = {
                hide = false,
            },
            dungeons = {}
        }
    })    

    self.db.RegisterCallback(self, "OnNewProfile", "SetConfigDefaults")    

    if LDB and LDBIcon then
        LDBIcon:Register(_G["addonName"], AISYLauncher, self.db.profile.minimap)
    end

    addon:RegisterEvent("UPDATE_INSTANCE_INFO", "UPDATE_INSTANCE_INFO")
    RequestRaidInfo()

    self:SetUpConfig()
end

function addon:SetUpConfig()
    local AceConfig = LibStub("AceConfig-3.0")
    local myOptionsTable = {
        type = "group",
        args = {
            showMiniMap = {
                name = "Show Minimap",  -- TODO: Localized 
                desc = "Show Minimap Icon",  -- TODO: Localized 
                type = "toggle",
                set = function(info, val) 
                    self.db.profile.minimap.hide = not val 
                    if val then 
                        LDBIcon:Show(_G["addonName"]) 
                    else 
                        LDBIcon:Hide(_G["addonName"]) 
                    end
                end,
                get = function(info) return not self.db.profile.minimap.hide end
            },
            dungeons = {
                name = "Dungeons", -- TODO: Localized
                desc = "Dungeons to monitor", -- TODO: Localized
                type = "group",
                args = {
                }
            }
        }
    }
    local counter = 1
    myOptionsTable.args.dungeons.args["header"] = {
        type = "header",
        name = "Current monitored dungeons, clear to delete", -- TODO: Localized
        order = 0
    }

    for key,_ in pairs(self.db.profile.dungeons) do
        myOptionsTable.args.dungeons.args[key] = {
            type = "input",
            name = "",
            desc = "",
            order = counter,
            set = function (info, val)
                if val == "" or val ~= info[2] then
                    addon:RemoveDungeon(info[2])
                elseif val ~= info[2] then
                    addon:AddDungeon(val)
                end
                addon:UpdateConfig()                
            end,
            get = function (info)
                return info[2] -- dangerous assumption I know, but it seems to work this way
            end
        }
        counter = counter + 1
    end

    myOptionsTable.args.dungeons.args["headernewdungeon"] = {
        type = "header",
        name = "New dungeon", -- TODO: Localized
        order = counter
    }
    counter = counter + 1

    myOptionsTable.args.dungeons.args["newdungeon"] = { 
        type = "input",
        name = "",
        order = counter,
        set = function (info, val)
            if val ~= "" then
                addon:AddDungeon(val)
                addon:UpdateConfig()
            end
        end,
        get = function (info)
            return ""
        end
    }    

    AceConfig:RegisterOptionsTable(_G["addonName"], myOptionsTable)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(_G["addonName"], _G["addonDesc"])
end


function addon:GetMonitoredDungeons()
    return self.db.profile.dungeons
end

function addon:ResetDungeons()
    for key,_ in pairs(self.db.profile.dungeons) do
        self.db.profile.dungeons[key] = true
    end    
end    

function addon:UPDATE_INSTANCE_INFO()
    self:ResetDungeons()
    for i=1,GetNumSavedInstances(),1 do
        local instanceName, id, reset, diff = GetSavedInstanceInfo(i)
        local diffName = GetDifficultyInfo(diff)
        if diffName == "Mythic" and self.db.profile.dungeons[instanceName] and reset > 0 then  -- TODO: Localized 
            self.db.profile.dungeons[instanceName] = false
        end
    end    
end

function addon:RemoveDungeon(dungeon)
    self.db.profile.dungeons[dungeon] = nil
end

function addon:AddDungeon(dungeon)
    self.db.profile.dungeons[dungeon] = true
end

function addon:UpdateConfig()
    addon:SetUpConfig()
    addon:ResetDungeons()
    RequestRaidInfo()
end

function addon:SetConfigDefaults(db, profile)    
    self.db.profile.dungeons = _G["defaultDungeons"]
end